ScriptName magicSoulTrapFXScript extends ActiveMagicEffect
{Scripted effect for the Soul Trap Visual FX}

import debug
import StringUtil

;======================================================================================;
;	PROPERTIES	/
;=============/
ImageSpaceModifier property TrapImod auto
{IsMod applied when we trap a soul}
sound property TrapSoundFX auto ; create a sound property we'll point to in the editor
{Sound played when we trap a soul}
VisualEffect property TargetVFX auto
{Visual Effect on Target aiming at Caster}
VisualEffect property CasterVFX auto
{Visual Effect on Caster aiming at Target}
EffectShader property CasterFXS auto
{Effect Shader on Caster during Soul trap}
EffectShader property TargetFXS auto
{Effect Shader on Target during Soul trap}
bool property bIsEnchantmentEffect = false auto
{Set this to true if this soul trap is on a weapon enchantment or a spell that can do damage to deal with a fringe case}

;======================================================================================;
;	VARIABLES	/
;=============/
actor victim
bool DeadAlready = false
bool bUseWait = true

;======================================================================================;
; ASG VARIABLES		/
;=================/
string ASGversion = "4.0"

; Handles thread locking so only one Soul Trap occurs at a time.
ASGThreadLock ThreadLockHandler
ObjectReference ASGGemHolderRef
GlobalVariable ASGOnlyBlack		; 000d63
GlobalVariable ASGOnlyFull		; 000d64
GlobalVariable ASGSilent		; 000d65
Keyword ActorTypeNPC			; 013794
SoulGem SoulGemLesser			; 02e4e4
SoulGem SoulGemCommon			; 02e4e6
SoulGem SoulGemGreater			; 02e4f4
SoulGem SoulGemGrand			; 02e4fc
SoulGem SoulGemBlack			; 02e500
SoulGem DA01SoulGemAzurasStar	; 063b27
SoulGem DA01SoulGemBlackStar	; 063b29

int numBlackStar = 0
int numAzurasStar = 0
int numBlack = 0
int numGrand = 0
int numGreater = 0
int numCommon = 0
int numLesser = 0
;======================================================================================;
;	EVENTS		/
;=============/

Event OnEffectStart(Actor Target, Actor Caster)
	debug.trace("ASGSoulTrap v"+ASGversion+" - Beginning Effect")
	victim = target
	
	if bIsEnchantmentEffect == False
		DeadAlready = victim.IsDead()
	endif
	bUseWait = False
	if (!victim.isDead() && victim.getActorValue("BrainCondition") == 8675309)
		victim.setActorValue("BrainCondition", 100)
	endif
	;debug.trace("Is Soultrap target dead? ("+deadAlready+")("+victim+")") 
endEvent

Event OnEffectFinish(Actor Target, Actor Caster)
	debug.trace("ASGSoulTrap v"+ASGversion+" - Ending Effect")
	;debug.trace(self + " is finishing")

	;setup lock
	ThreadLockHandler = game.getFormFromFile(0x0022EE, "AcquisitiveSoulGemMultithreaded.esp") as ASGThreadLock
	if ThreadLockHandler != None
		debug.trace("ASGSoulTrap v"+ASGversion+" - Acquiring Lock")
		while !ThreadLockHandler.AcquireLock(Caster)
			utility.wait(0.1)
;			debug.trace("USKP testMessage: magicSoulTrapFXScript Looping (waiting for permission)....")
		endWhile
		debug.trace("ASGSoulTrap v"+ASGversion+" - Lock Acquired")
	endif

	if victim
		if bUseWait 
			Utility.Wait(0.25)
		endif
		
		debug.trace("ASGSoulTrap v"+ASGversion+" - BrainCondition: "+victim.getActorValue("BrainCondition"))
		if (!DeadAlready && victim.isDead() && victim.getActorValue("BrainCondition") != 8675309)
			debug.trace("ASGSoulTrap v"+ASGversion+" - BrainCondition is good, proceeding")
			;set references
			ASGGemHolderRef = Game.GetFormFromFile(0x002857, "AcquisitiveSoulGemMultithreaded.esp") as ObjectReference
			ActorTypeNPC = Game.GetFormFromFile(0x013794, "Skyrim.esm") as Keyword
			ASGOnlyBlack = Game.GetFormFromFile(0x000d63, "AcquisitiveSoulGemMultithreaded.esp") as GlobalVariable
			ASGOnlyFull = Game.GetFormFromFile(0x00d64, "AcquisitiveSoulGemMultithreaded.esp") as GlobalVariable
			ASGSilent = Game.GetFormFromFile(0x000d65, "AcquisitiveSoulGemMultithreaded.esp") as GlobalVariable
			SoulGemLesser = Game.GetFormFromFile(0x02e4e4, "Skyrim.esm") as SoulGem
			SoulGemCommon = Game.GetFormFromFile(0x02e4e6, "Skyrim.esm") as SoulGem
			SoulGemGreater = Game.GetFormFromFile(0x02e4f4, "Skyrim.esm") as SoulGem
			SoulGemGrand = Game.GetFormFromFile(0x02e4fc, "Skyrim.esm") as SoulGem
			SoulGemBlack = Game.GetFormFromFile(0x02e500, "Skyrim.esm") as SoulGem
			DA01SoulGemAzurasStar = Game.GetFormFromFile(0x063b27, "Skyrim.esm") as SoulGem
			DA01SoulGemBlackStar = Game.GetFormFromFile(0x063b29, "Skyrim.esm") as SoulGem

			numBlackStar = Caster.GetItemCount(DA01SoulGemBlackStar)
			numAzurasStar = Caster.GetItemCount(DA01SoulGemAzurasStar)

			;perform soul size checks
			int targetLevel = victim.GetLevel()
			debug.trace("ASGSoulTrap v"+ASGversion+" - Performing Gem checks")
			string soulType = "Black"
			if(!victim.HasKeyword(ActorTypeNPC))
				if(ASGOnlyBlack.GetValueInt() == 1)
					Caster.RemoveItem(DA01SoulGemBlackStar, numBlackStar, true, ASGGemHolderRef)
					numBlack = Caster.GetItemCount(SoulGemBlack)
					Caster.RemoveItem(SoulGemBlack, numBlack, true)
				endif
				soulType = "Grand"
				if(targetLevel < Game.GetGameSettingInt("iGrandSoulActorLevel"))
					numBlack = numBlack + Caster.GetItemCount(SoulGemBlack)
					Caster.RemoveItem(SoulGemBlack, numBlack, true )
					numGrand = Caster.GetItemCount(SoulGemGrand)
					Caster.RemoveItem(SoulGemGrand, numGrand, true )
					if(ASGOnlyFull.GetValueInt() == 1)
						Caster.RemoveItem(DA01SoulGemBlackStar, numBlackStar, true, ASGGemHolderRef)
						Caster.RemoveItem(DA01SoulGemAzurasStar, numAzurasStar, true, ASGGemHolderRef)
					endif
					soulType = "Greater"
				endif
				if(targetLevel < Game.GetGameSettingInt("iGreaterSoulActorLevel"))
					numGreater = Caster.GetItemCount(SoulGemGreater)
					Caster.RemoveItem(SoulGemGreater, numGreater, true )
					soulType = "Common"
				endif
				if(targetLevel < Game.GetGameSettingInt("iCommonSoulActorLevel"))
					numCommon = Caster.GetItemCount(SoulGemCommon)
					Caster.RemoveItem(SoulGemCommon, numCommon, true )
					soulType = "Lesser"
				endif
				if(targetLevel < Game.GetGameSettingInt("iLesserSoulActorLevel"))
					numLesser = Caster.GetItemCount(SoulGemLesser)
					Caster.RemoveItem(SoulGemLesser, numLesser, true )
					soulType = "Petty"
				endif
			endif
			
			;capture soul
			debug.trace("ASGSoulTrap v"+ASGversion+" - Trapping soul")
			bool result = Caster.TrapSoul(victim)
			victim.setActorValue("BrainCondition", 8675309)

			;return all gems
			debug.trace("ASGSoulTrap v"+ASGversion+" - Returning Gems")
			ASGGemHolderRef.RemoveItem(DA01SoulGemBlackStar, numBlackStar, true, Caster)
			ASGGemHolderRef.RemoveItem(DA01SoulGemAzurasStar, numAzurasStar, true, Caster)
			Caster.addItem(SoulGemBlack, numBlack, true )
			Caster.addItem(SoulGemGrand, numGrand, true )
			Caster.addItem(SoulGemGreater, numGreater, true )
			Caster.addItem(SoulGemCommon, numCommon, true )
			Caster.addItem(SoulGemLesser, numLesser, true )
			
			;custom message
			debug.trace("ASGSoulTrap v"+ASGversion+" - Vanilla handling started")
			if(result)
				
				; Vanilla: "You have captured a soul!"

				;English-only non-SKSE version
				if(ASGSilent.GetValueInt() == 0)
					debug.notification(soulType+" soul captured!")
				endif
				;International SKSE version
				if(ASGSilent.GetValueInt() == 2)
					debug.notification("$ASGCaptured"+soulType)
				endif
				;debug.trace(victim + " is, in fact, dead.	Play soul trap visFX")
				TrapSoundFX.play(Caster)			 ; play TrapSoundFX sound from player
				TrapImod.apply()																	; apply isMod at full strength
				TargetVFX.Play(victim,4.7,Caster)							; Play TargetVFX and aim them at the player
				CasterVFX.Play(Caster,5.9,victim)
				TargetFXS.Play(victim,2)								; Play Effect Shaders
				CasterFXS.Play(Caster,3)
			else
				;debug.trace(victim + " is, in fact, dead, But the TrapSoul check failed or came back false")
				; Vanilla: "There is no Soul Gem large enough to capture the soul."
				
				;English-only non-SKSE version
				if(ASGSilent.GetValueInt() == 0)
					debug.notification("Failed to capture "+soulType+" soul.")
				endif
				;International SKSE version
				if(ASGSilent.GetValueInt() == 2)
					debug.notification("$ASGFailed"+soulType)
				endif
			endif
		;else
			;debug.trace(self + "tried to soulTrap, but " + victim + " is already Dead.")
		endif
	endif

	;release lock
	if ThreadLockHandler != None
		debug.trace("ASGSoulTrap v"+ASGversion+" - Releasing Lock")
		ThreadLockHandler.ReleaseLock(Caster)
		debug.trace("ASGSoulTrap v"+ASGversion+" - Lock Released")
	endif
	debug.trace("ASGSoulTrap v"+ASGversion+" - Effect Ended")
endEvent
